from src.breached_password import password_breached


# Test Breached Password


def test_password_breached():
    assert password_breached('abcd') == "The password was found in compromised password lists!"


test_password_breached()




import hashlib
import os


# Password Hash


class HashPassword:

    def __init__(self):
        self.salt = os.urandom(16)


    def hash_password(self, password):
        salted_password = self.salt + password.encode('utf-8')
        hashed_password = hashlib.sha256(salted_password).hexdigest()
        return (hashed_password, self.salt)


    def verify_password(self, password, salt, hashed_password):
        salted_password = salt + password.encode('utf-8')
        new_hashed_password = hashlib.sha256(salted_password).hexdigest()
        if new_hashed_password == hashed_password:
            return True
        else:
            return False




import requests
import hashlib


# Check Breached Password


def password_breached(password):
    sha1_password = hashlib.sha1(password.encode('utf-8')).hexdigest().upper()
    prefix = sha1_password[:5]
    suffix = sha1_password[5:]
    url = f"https://api.pwnedpasswords.com/range/{prefix}"
    response = requests.get(url)
    if response.status_code != 200:
        raise ValueError("Error fetching breached passwords")
    for line in response.content.decode('utf-8').splitlines():
        line_suffix, count = line.split(':')
        if line_suffix == suffix:
            return "The password was found in compromised password lists!"
    return True




from src.suggest_improvements import improvements_suggest


# Test Suggest Password Improvements


def test_improvements_suggest():
    assert improvements_suggest('Abcd1234') == True
    assert improvements_suggest('Abc123')[0] == "Password should be at least 8 characters long."
    assert improvements_suggest('abcde1234')[0] == "Password should contain at least one uppercase letter."
    assert improvements_suggest('ABCDE1234')[0] == "Password should contain at least one lowercase letter."
    assert improvements_suggest('Abcdefgeh')[0] == "Password should contain at least one digit."


test_improvements_suggest()




import random
import string


# Suggest Password Improvements


def improvements_suggest(password):

    improvements = []

    if len(password) < 8:
        improvements.append("Password should be at least 8 characters long.")

    if not any(char.isupper() for char in password):
        improvements.append("Password should contain at least one uppercase letter.")

    if not any(char.islower() for char in password):
        improvements.append("Password should contain at least one lowercase letter.")

    if not any(char.isdigit() for char in password):
        improvements.append("Password should contain at least one digit.")

    if len(improvements) == 0:
        return True
    
    else:
        random_chars = ''.join(random.choices(string.ascii_letters + string.digits, k=4))
        new_password = password + random_chars
        improvements.append(new_password)

        return improvements
    



from src.password_generation import generate_password


# Test Password Generation


def test_generate_password():
    assert generate_password('Hello i am benny and i like @') == False


test_generate_password()




from src.password_hash import HashPassword


# Test Password Hash


def test_hash_password():
    hasher = HashPassword()

    hashed_password, salt = hasher.hash_password('Abcde1234')

    assert hasher.verify_password('wrong_password', salt, hashed_password) == False


test_hash_password()




from src.strength_checker import check_strength


# Test Password Strength Checker


def test_check_strength():
    assert check_strength('Abcdefg123') == True
    assert check_strength('abc123') == "Password is too short. It should be at least 8 characters long."
    assert check_strength('abcdefgh') == "Password should contain at least one uppercase letter."
    assert check_strength('ABCDEFGH') == "Password should contain at least one lowercase letter."


test_check_strength()




import pyotp


# Multi Factor Authentication


class MultiFactorAuth:

    def __init__(self, secret_key=None):
        if secret_key is None:
            self.secret_key = pyotp.random_base32()
        else:
            self.secret_key = secret_key

    
    def generate_otp(self):
        totp = pyotp.TOTP(self.secret_key)
        return totp.now()
    

    def verify_otp(self, otp):
        totp = pyotp.TOTP(self.secret_key)
        return totp.verify(otp)
    



import re


# Password Strength Checker


def check_strength(password):
    
    if len(password) < 8:
        return "Password is too short. It should be at least 8 characters long."
    
    if not re.search(r'[A-Z]', password):
        return "Password should contain at least one uppercase letter."
    
    if not re.search(r'[a-z]', password):
        return "Password should contain at least one lowercase letter."
    
    if not re.search(r'\d', password):
        return "Password should contain at least one digit."
    
    return True
    



from src.multi_factor_authetication import MultiFactorAuth


# Test Multi Factor Authentication


def test_multi_factor_auth():
    mfa = MultiFactorAuth()
    otp = mfa.generate_otp()
    assert mfa.verify_otp('123456') == False
    assert mfa.verify_otp(otp) == True


test_multi_factor_auth()




import random
import string
import re


# Password Generation


def generate_password(sentence):
    if re.search(r'[!@#$%^&*(),.?":{}|<>]', sentence):
        return False
    
    words = sentence.split()
    password = ""
    password += words[0]
    for i in range(1, len(words)):
        if len(words[i]) >= 4:
            password += words[i][:4]
        else:
            password += words[i]
    password += "".join(random.choices(string.digits, k=4))
    
    while len(password) < 8 or not any(char.isupper() for char in password) or not any(char.islower() for char in password) or not any(char.isdigit() for char in password):
        password += random.choice(string.ascii_letters + string.digits)

    return password

